@@ -3,17 +3,13 @@ use Modern::Perl;
 use Carp;
 use Graph;
 use Audio::Nama::Util qw(input_node output_node);
-use Log::Log4perl;
-use Audio::Nama::Log qw(logsub);
+use Audio::Nama::Log qw(logsub logpkg);
+use Audio::Nama::Globals qw(:trackrw);
 use vars qw(%reserved);
 # this dispatch table also identifies labels reserved
 # for signal sources and sinks.
 *reserved = \%Audio::Nama::IO::io_class;
 
-our $logger;
-
-sub initialize_logger { $logger = Log::Log4perl->get_logger() }
-
 sub add_path_for_rec {
 
 	# connect input source to file 
@@ -35,11 +31,11 @@ sub add_path_for_rec {
 		# temporary track by providing the data as 
 		# graph edge attributes)
 
-		$logger->debug("rec file link for ".$track->name);
+		logpkg(__FILE__,__LINE__,'debug',"rec file link for ".$track->name);
 		my $name = $track->name . '_rec_file';
 		my $anon = Audio::Nama::SlaveTrack->new( 
 			target => $track->name,
-			rw => 'OFF',
+			rw => OFF,
 			group => 'Temp',
 			hide => 1,
 			name => $name);
@@ -76,7 +72,6 @@ sub add_path_for_rec {
 
 		# generally a sub bus 
 
-		# - has 'rec_defeat' set (therefore doesn't reach here)
 		# - receives a stereo input
 		# - mix track width is set to stereo (default)
 
@@ -112,6 +107,7 @@ sub add_path_for_send {
 	my ($g, $name, $send_type, $send_id)  = @_;
 
 	logsub("&add_path_for_aux_send: track ".$name);
+	logpkg(__FILE__,__LINE__,'debug',"args: graph: $g, name: $name, send_type, $send_type, send_id: $send_id");
 
 	# for track 'sax', send_type 'track' send_id 'vocal'
 	#
@@ -121,20 +117,21 @@ sub add_path_for_send {
 		@path = ($name, $send_id)
 	}
 	else {
-
+		
 		# for track 'sax', send_type 'jack_client', create route as 
 		# sax -> sax_aux_send -> jack_client_out
 		
 		my $nameof = $name . '_aux_send';
 		my $anon = Audio::Nama::SlaveTrack->new( 
 			target => $name,
-			rw => 'OFF',
+			rw => OFF,
 			group => 'Temp',
 			hide => 1,
 			name => $nameof);
 
 		@path= ($name, $nameof, output_node($send_type));
 	}
+	logpkg(__FILE__,__LINE__,'debug',"adding path ", join '-',@path);
 	$g->add_path(@path);
 }
 {
@@ -148,28 +145,28 @@ sub expand_graph {
 	
 	for ($g->edges){
 		my($a,$b) = @{$_}; 
-		$logger->debug("$a-$b: processing...");
-		$logger->debug("$a-$b: already seen") if $seen{"$a-$b"};
+		logpkg(__FILE__,__LINE__,'debug',"$a-$b: processing...");
+		logpkg(__FILE__,__LINE__,'debug',"$a-$b: already seen") if $seen{"$a-$b"};
 		next if $seen{"$a-$b"};
 
 		# case 1: both nodes are tracks: default insertion logic
 	
 		if ( is_a_track($a) and is_a_track($b) ){ 
-			$logger->debug("processing track-track edge: $a-$b");
+			logpkg(__FILE__,__LINE__,'debug',"processing track-track edge: $a-$b");
 			add_loop($g,$a,$b) } 
 
 		# case 2: fan out from track: use near side loop
 
 		elsif ( is_a_track($a) and $g->successors($a) > 1 ) {
-			$logger->debug("fan_out from track $a");
+			logpkg(__FILE__,__LINE__,'debug',"fan_out from track $a");
 			add_near_side_loop($g,$a,$b,out_loop($a));}
 	
 		# case 3: fan in to track: use far side loop
 		
 		elsif ( is_a_track($b) and $g->predecessors($b) > 1 ) {
-			$logger->debug("fan in to track $b");
+			logpkg(__FILE__,__LINE__,'debug',"fan in to track $b");
 			add_far_side_loop($g,$a,$b,in_loop($b));}
-		else { $logger->debug("$a-$b: no action taken") }
+		else { logpkg(__FILE__,__LINE__,'debug',"$a-$b: no action taken") }
 	}
 	
 }
@@ -187,11 +184,11 @@ sub add_inserts {
 
 sub add_loop {
 	my ($g,$a,$b) = @_;
-	$logger->debug("adding loop");
+	logpkg(__FILE__,__LINE__,'debug',"adding loop");
 	my $fan_out = $g->successors($a);
-	$logger->debug("$a: fan_out $fan_out");
+	logpkg(__FILE__,__LINE__,'debug',"$a: fan_out $fan_out");
 	my $fan_in  = $g->predecessors($b);
-	$logger->debug("$b: fan_in $fan_in");
+	logpkg(__FILE__,__LINE__,'debug',"$b: fan_in $fan_in");
 	if ($fan_out > 1){
 		add_near_side_loop($g,$a,$b, out_loop($a))
 	} elsif ($fan_in  > 1){
@@ -245,14 +242,9 @@ sub add_loop {
 #  with, for example, multiple edges to a soundcard.
 #
 #  Send buses create new tracks to provide connections.
-#
-# I will be moving edges (along with their attributes)
-# but I cannot assign chain_id them because I have
-# no way of knowing which is the edge that will use
-# the track number and will therefore get the track effects
 
  	my ($g, $a, $b, $loop) = @_;
- 	$logger->debug("$a-$b: insert near side loop");
+ 	logpkg(__FILE__,__LINE__,'debug',"$a-$b: insert near side loop");
 	# we will insert loop _after_ processing successor
 	# edges so $a-$loop will not be picked up 
 	# in successors list.
@@ -271,7 +263,7 @@ sub add_loop {
 		track => $Audio::Nama::tn{$a}->name});
 	map{ 
  		my $attr = $g->get_edge_attributes($a,$_);
- 		$logger->debug("deleting edge: $a-$_");
+ 		logpkg(__FILE__,__LINE__,'debug',"deleting edge: $a-$_");
  		$g->delete_edge($a,$_);
 		$g->add_edge($loop, $_);
 		$g->set_edge_attributes($loop,$_, $attr) if $attr;
@@ -283,14 +275,14 @@ sub add_loop {
 
 sub add_far_side_loop {
  	my ($g, $a, $b, $loop) = @_;
- 	$logger->debug("$a-$b: insert far side loop");
+ 	logpkg(__FILE__,__LINE__,'debug',"$a-$b: insert far side loop");
 	
 	$g->set_vertex_attributes($loop,{
 		n => $Audio::Nama::tn{$a}->n, j => 'a',
 		track => $Audio::Nama::tn{$a}->name});
 	map{ 
  		my $attr = $g->get_edge_attributes($_,$b);
- 		$logger->debug("deleting edge: $_-$b");
+ 		logpkg(__FILE__,__LINE__,'debug',"deleting edge: $_-$b");
  		$g->delete_edge($_,$b);
 		$g->add_edge($_,$loop);
 		$g->set_edge_attributes($_,$loop, $attr) if $attr;
@@ -319,11 +311,10 @@ sub inputless_tracks {
 }	
 sub remove_out_of_bounds_tracks {
 	my $g = shift;
-	my @names = $g->successors('wav_in');  # MON status tracks
+	my @names = $g->successors('wav_in');  # PLAY status tracks
 	map{ remove_tracks($g, $_) } 
 	grep{
-		Audio::Nama::set_edit_vars($Audio::Nama::tn{$_});
-		Audio::Nama::edit_case() =~ /out_of_bounds/
+		Audio::Nama::edit_case(Audio::Nama::edit_vars($Audio::Nama::tn{$_})) =~ /out_of_bounds/
 	} @names;
 }
 
@@ -367,5 +358,21 @@ sub remove_isolated_vertices {
 	grep{ $g->is_isolated_vertex($_) } $g->vertices();	
 }
 
+sub simplify_send_routing {
+	my $g = shift;
+	for( grep { is_a_track($_) } $g->vertices ){
+		my $aux = "$_\_aux_send";
+		my @successors;
+		if( $g->has_edge($_, $aux)
+			and @successors = $g->successors($_)
+			and scalar @successors == 1
+		){
+			my ($output) = $g->successors($aux);
+			$g->delete_path($_, $aux, $output);
+			$g->add_edge($_, $output);
+		}	
+	}
+}
+
 1;
 __END__
\ No newline at end of file
