@@ -5,7 +5,7 @@ package Audio::Nama::Project;
 use Modern::Perl; use Carp;
 sub hello { my $self = shift; say "hello $self: ",Audio::Nama::Dumper $Audio::Nama::project}
 }
-
+{
 package Audio::Nama;
 use Modern::Perl;
 use Carp;
@@ -36,7 +36,17 @@ sub project_dir {
 	$config->{opts}->{p} and return $config->{root_dir}; # cwd
 	$project->{name} and join_path( project_root(), $project->{name}) 
 }
+sub this_op { $this_track and $this_track->op }
+sub this_op_o { $this_track and $this_track->op and fxn($this_track->op) }
+sub this_param { $this_track ? $this_track->param : ""}
+sub this_stepsize { $this_track ? $this_track->stepsize : ""}
+sub this_track_name { $this_track ? $this_track->name : "" }
 
+# we prepend a slash 
+sub bus_track_display { 
+	my ($busname, $trackname) = ($this_bus, $this_track && $this_track->name || '');
+	($busname eq "Main" ? "": "$busname/" ). $trackname
+}
 sub list_projects {
 	my $projects = join "\n", sort map{
 			my ($vol, $dir, $lastdir) = File::Spec->splitpath($_); $lastdir
@@ -63,29 +73,27 @@ sub initialize_project_data {
 	$gui->{_markers_armed} = 0;
 
 	map{ $_->initialize() } qw(
-							Audio::Nama::Mark
-							Audio::Nama::Fade
 							Audio::Nama::Edit
+							Audio::Nama::Fade
+							Audio::Nama::Mark
 							Audio::Nama::Bus
 							Audio::Nama::Track
 							Audio::Nama::Insert
+							Audio::Nama::Engine
 							);
 	initialize_effects_data();
 
 	# $is_armed = 0;
 
-	$setup->{_old_snapshot} = {};
+	$setup->{_old_snapshot} = "";
 
-	$mode->{preview} = $config->{initial_mode};
 	$mode->{mastering} = 0;
 
 	$project->{save_file_version_number} = 0; 
 	$project->{track_comments} = {};
 	$project->{track_version_comments} = {};
-	$project->{undo_buffer} = [];
 	$project->{repo} = undef;
 	$project->{artist} = undef;
-	
 	$project->{bunch} = {};	
 	
 	create_system_buses();
@@ -97,9 +105,11 @@ sub initialize_project_data {
 	$mode->{offset_run} = 0;
 	$this_edit = undef;
 	
-	$mode->{preview} = 0;
+	$mode->{preview} = $config->{initial_mode};
 
 	Audio::Nama::ChainSetup::initialize();
+	reset_hotkey_buffers();
+	reset_command_buffer();
 
 }
 sub initialize_effects_data {
@@ -118,8 +128,6 @@ sub initialize_effects_data {
 
 }
 
-	
-
 sub load_project {
 	logsub("&load_project");
 	my %args = @_;
@@ -136,7 +144,7 @@ sub load_project {
 			map{create_dir($_)} project_dir(), this_wav_dir() ;
 		}
 		else 
-		{ Audio::Nama::pager3(
+		{ Audio::Nama::pager_newline(
 			qq(Project "$project->{name}" does not exist.\n Loading project "untitled".)
 			);
 			load_project( qw{name untitled create 1} );
@@ -165,13 +173,12 @@ sub load_project {
 
 		if ($initializing_repo){
 			$project->{repo}->run( add => $file->git_state_store );
-			$project->{repo}->run( commit => '--quiet', '--message', "initial commit");
 		}
 	}
 
 	restore_state($args{settings}) unless $config->{opts}->{M} ;
 
-	if (! $tn{Master}){
+	if (! $tn{Master}){ # new project
 
 		Audio::Nama::SimpleTrack->new( 
 			group => 'Master', 
@@ -179,8 +186,7 @@ sub load_project {
 			send_type => 'soundcard',
 			send_id => 1,
 			width => 2,
-			rw => 'REC',
-			rec_defeat => 1,
+			rw => MON,
 			source_type => undef,
 			source_id => undef); 
 
@@ -188,7 +194,7 @@ sub load_project {
 			group => 'Mixdown', 
 			name => 'Mixdown', 
 			width => 2,
-			rw => 'OFF',
+			rw => OFF,
 			source_type => undef,
 			source_id => undef); 
 
@@ -203,6 +209,8 @@ sub load_project {
 	# $args{nodig} allow skip for convert_project_format
 	dig_ruins() unless (scalar @Audio::Nama::Track::all > 2 ) or $args{nodig};
 
+	git_commit("initialize new project") if $config->{use_git};
+
 	# possible null if Text mode
 	
 	#$ui->global_version_buttons(); 
@@ -227,7 +235,7 @@ sub restore_state {
 sub dig_ruins { # only if there are no tracks 
 	
 	logsub("&dig_ruins");
-	return if Audio::Nama::Track::user();
+	return if user_tracks_present();
 	logpkg(__FILE__,__LINE__,'debug', "looking for WAV files");
 
 	# look for wave files
@@ -318,17 +326,17 @@ sub create_system_buses {
 sub new_project_template {
 	my ($template_name, $template_description) = @_;
 
-	my @tracks = Audio::Nama::Track::all();
+	my @tracks = all_tracks();
 
 	# skip if project is empty
 
 	throw("No user tracks found, aborting.\n",
 		"Cannot create template from an empty project."), 
-		return if scalar @tracks < 3;
+		return if ! user_tracks_present();
 
 	# save current project status to temp state file 
 	
-	my $previous_state = '_previous_state.yml';
+	my $previous_state = '_previous_state.json';
 	save_state($previous_state);
 
 	# edit current project into a template
@@ -337,7 +345,6 @@ sub new_project_template {
 	#	- version (still called 'active')
 	# 	- track caching
 	# 	- region start/end points
-	# 	- effect_chain_stack
 	# Also
 	# 	- unmute all tracks
 	# 	- throw away any pan caching
@@ -350,9 +357,6 @@ sub new_project_template {
 				region_start
 				region_end
 			);
-		 map{ $track->set($_ => [])  } 
-			qw(	effect_chain_stack  );
-		
 	} @tracks;
 
 	# Throw away command history
@@ -369,7 +373,7 @@ sub new_project_template {
 
 	# save to template name
 	
-	save_state( join_path(project_root(), "templates", "$template_name.yml"));
+	save_state( join_path(project_root(), "templates", "$template_name.json"));
 
 	# add description, but where?
 	
@@ -382,7 +386,7 @@ sub new_project_template {
 
 	# remove temp state file
 	
-	unlink join_path( project_dir(), "$previous_state.yml") ;
+	unlink join_path( project_dir(), $previous_state) ;
 	
 }
 sub use_project_template {
@@ -398,22 +402,23 @@ sub use_project_template {
 	
  	load_project(
  		name     => $project->{name},
- 		settings => join_path(project_root(),"templates",$name),
+ 		settings => join_path(project_root(),"templates","$name.json"),
 	);
 	save_state();
 }
 sub list_project_templates {
-	my $read = read_file(join_path(project_root(), "templates"));
-	push my @templates, "\nTemplates:\n", map{ m|([^/]+).yml$|; $1, "\n"} $read;        
-	pager(@templates);
+	my @templates= map{ /(.+?)\.json$/; $1}  read_dir(join_path(project_root(), "templates"));
+	
+	pager(join "\n","Templates:",@templates);
 }
 sub remove_project_template {
 	map{my $name = $_; 
-		pager2("$name: removing template");
+		pager("$name: removing template");
 		$name .= ".yml" unless $name =~ /\.yml$/;
 		unlink join_path( project_root(), "templates", $name);
 	} @_;
 	
 }
+} # end package ::
 1;
 __END__
\ No newline at end of file
